<?php

// If this file is called directly, abort.
if (!defined('FLC_BASEPATH')) {
    die;
}

class FLC_API {

    use FLC_Singleton;

    protected $key      = '';
    protected $secret   = '';

    protected $working  = null;

    protected function __construct() {

        $this->key = FLC_Settings::instance()->get('api_key', $this->key);
        $this->secret = FLC_Settings::instance()->get('api_secret', $this->secret);
    }

    public function getKey() {
        
        return $this->key;
    }

    public function getSecret() {
        
        return $this->secret;
    }

    public function call($api, $params = array()) {
        
        $address = FLC_FormaLMS::instance()->getAddress();
        $token = base64_encode($this->key . ':' . sha1(implode(',', $params) . ',' . $this->secret));

        $result = wp_remote_post("$address/api/rest.php?q=/api/$api", array(
            'headers'   => array(
                "Accept"            => "application/json"
              , "X-Authorization"   => "FormaLMS $token"
            )
          , 'body'      => $params
          , 'sslverify' => false
        ));
        
        if(is_wp_error($result)) {
            $response = $result->errors;
        } else if($result['response']['code'] != 200) {
            $response = $result['response'];
        } else {
            $response = json_decode($result['body'], true);
            if(json_last_error() !== JSON_ERROR_NONE) {
                $response = array('error' => "Response body is not a valid JSON.");
            } else if(empty($response)) {
                $response = array('error' => "Response body is empty.");
            }
        }

        $this->writeLog($api, $params, $response);

        return $response;
    }
    
    private function writeLog($api, $params, $response) {
        
        $datetime = date('Y-m-d H:i:s');
        
        $api        = json_encode($api, JSON_UNESCAPED_SLASHES);
        $params     = json_encode($params, JSON_UNESCAPED_SLASHES);
        $response   = json_encode($response, JSON_UNESCAPED_SLASHES);
        
        $text = "$datetime --- API: $api - PARAMS: $params - RESPONSE: $response\n";
        
        file_put_contents($this->getLogFolder() . date('Ym_') . 'api.log', $text, FILE_APPEND);
    }

    public function getLogFolder() {

        $upload_dir_arr = wp_upload_dir();
        $path = $upload_dir_arr['basedir'] . '/flc/logs/';

        return $path;
    }

    public function createLogFolder() {
        
        @mkdir($this->getLogFolder(), 0777, true);
    }

    public function deleteLogFolder() {
        
        $dir = $this->getLogFolder();
        foreach(scandir($dir) as $file) {
            if(is_file("$dir/$file")) {
                @unlink("$dir/$file");
            }
        }
        @rmdir($this->getLogFolder());
    }

    public function test($force = false) {

        if(is_null($this->working)) {
            $this->working = FLC_Settings::instance()->get('api_working', $this->working);
            if(is_null($this->working) || $force) {
                $res = $this->call('user/countusers');
                $this->working = !empty($res['success']);
                FLC_Settings::instance()->set('api_working', $this->working);
            }
        }
        return $this->working;
    }
}
