<?php

// If this file is called directly, abort.
if (!defined('FLC_BASEPATH')) {
    die;
}

class FLC_SettingsPage {

    use FLC_Singleton;

    public function page() {

        add_settings_section(
            'flc_formalms_settings'
          , ''
          , array($this, 'sectionHTML')
          , 'flc_general_settings'
        );

        add_settings_field(
            'flc_version'
          , __('Version', 'flc-forma-lms-connector')
          , array($this, 'versionFieldHTML')
          , 'flc_general_settings'
          , 'flc_formalms_settings'
        );

        add_settings_field(
            'flc_address'
          , __('Address', 'flc-forma-lms-connector')
          , array($this, 'addressFieldHTML')
          , 'flc_general_settings'
          , 'flc_formalms_settings'
        );

        register_setting('flc_general_settings', 'flc_version');
        register_setting('flc_general_settings', 'flc_address');

        add_settings_section(
            'flc_api_settings'
          , __('API', 'flc-forma-lms-connector')
          , array($this, 'sectionHTML')
          , 'flc_general_settings'
        );

        add_settings_field(
            'flc_api_key'
          , __('Key', 'flc-forma-lms-connector')
          , array($this, 'APIKeyFieldHTML')
          , 'flc_general_settings'
          , 'flc_api_settings'
        );
        
        add_settings_field(
            'flc_api_secret'
          , __('Secret', 'flc-forma-lms-connector')
          , array($this, 'APISecretFieldHTML')
          , 'flc_general_settings'
          , 'flc_api_settings'
        );
        
        register_setting('flc_general_settings', 'flc_api_key');
        register_setting('flc_general_settings', 'flc_api_secret');

        foreach(FLC_Plugin::instance()->entities() as $entity => $class) {

            add_settings_section(
                'flc_entity-' . $entity . '_settings'
              , ''
              , array($this, 'sectionHTML')
              , 'flc_entity-' . $entity . '_settings'
            );
            
            $class::settings();
        }
    }

    public function pageHTML() {
        
        $messages = apply_filters('flc.settings.messages', array());

        $current_tab = isset($_GET['tab']) ? $_GET['tab'] : 'general';

        $tabs = array(
            'general' => __('General', 'flc-forma-lms-connector')
        );

        add_action('flc.settings.tab-general', function() {
            settings_fields('flc_general_settings');
            do_settings_sections('flc_general_settings');
        });

        foreach(FLC_Plugin::instance()->entities() as $entity => $class) {
            add_action('flc.settings.tab-entity-' . $entity, function() use ($entity) {
                settings_fields('flc_entity-' . $entity . '_settings');
                do_settings_sections('flc_entity-' . $entity . '_settings');
            });
            $tabs['entity-' . $entity] = $class::getTitle();
        }

        $tabs = apply_filters('flc.settings.tabs', $tabs);

        ?>
        <style type="text/css">
            form#flc_settings input[type="text"] {
                width: 300px;
            }
        </style>
        <div class="wrap">
            <h1 style="display: none;"><?php _e('Settings', 'flc-forma-lms-connector'); ?></h1><!-- required for notices -->
            <?php echo implode('', $messages); ?>
            <nav class="nav-tab-wrapper woo-nav-tab-wrapper">
            <?php foreach($tabs as $tab => $label): ?>
                <a href="<?php echo admin_url("admin.php?page=flc-settings&tab=$tab"); ?>" class="nav-tab<?php if($current_tab === $tab): ?> nav-tab-active<?php endif; ?>"><?php echo $label ?></a>
            <?php endforeach; ?>
            </nav>
            <form id="flc_settings" method="POST" action="options.php">
            <?php do_action("flc.settings.tab-$current_tab"); ?>
                <?php submit_button(); ?>
            </form>
        </div>
        <?php
    }

    public function sectionHTML() { /* Empty, it is not so usefull. */ }

    public function versionFieldHTML() {
        
        $selected = FLC_FormaLMS::instance()->getVersion();
        ?>
        <select name="flc_version">
        <?php foreach(array(
            '1.3' => __('1.3', 'flc-forma-lms-connector') 
          , '1.4' => __('1.4', 'flc-forma-lms-connector') 
          , '2.0' => __('2.x', 'flc-forma-lms-connector')
          , '3.0' => __('3.0 or greater', 'flc-forma-lms-connector')
        ) as $version_value => $version_label): ?>
            <option value="<?php echo $version_value ?>"<?php selected($selected === $version_value); ?>><?php echo $version_label; ?></option>
        <?php endforeach; ?>
        </select>
        <?php
    }

    public function addressFieldHTML() {
        
        $value = FLC_FormaLMS::instance()->getAddress();
        ?>
        <input type="text" name="flc_address" value="<?php echo esc_attr($value); ?>" />
        <?php
    }

    public function APIKeyFieldHTML() {
        
        $value = FLC_API::instance()->getKey();
        ?>
        <input type="text" name="flc_api_key" value="<?php echo esc_attr($value); ?>" />
        <?php
    }

    public function APISecretFieldHTML() {
        
        $value = FLC_API::instance()->getSecret();
        ?>
        <input type="text" name="flc_api_secret" value="<?php echo esc_attr($value); ?>" />
        <?php
    }

    public function messages($messages) {

        $address = FLC_FormaLMS::instance()->getAddress();
        $key = FLC_API::instance()->getKey();
        $secret = FLC_API::instance()->getSecret();

        if(!$address || !$key || !$secret) {
            $fields = array();
            if(!$address) {
                $fields[] = '-&emsp;' . __('Address', 'flc-forma-lms-connector');
            }
            if(!$key) {
                $fields[] = '-&emsp;' . __('Key', 'flc-forma-lms-connector');
            }
            if(!$secret) {
                $fields[] = '-&emsp;' . __('Secret', 'flc-forma-lms-connector');
            }
            $message = '<p>' . __('Missing required fields:', 'flc-forma-lms-connector') . '</p>';
            $message .= '<p>' . implode('<br />', $fields) . '</p>';
            $messages[$message] = "<div class='notice notice-warning is-dismissible'>$message</div>";
        } else {
            if(!FLC_API::instance()->test(isset($_GET['page']) && $_GET['page'] === 'flc-settings' && (!isset($_GET['tab']) || $_GET['tab'] === 'general'))) {
                $message = '<p>' . __('The system is unable to communicate with forma.lms. Please check your configuration.', 'flc-forma-lms-connector') . '</p>';
                $messages[$message] = "<div class='notice notice-error is-dismissible'>$message</div>";
            }
        }

        return $messages;
    }
}
