<?php

// If this file is called directly, abort.
if (!defined('FLC_BASEPATH')) {
    die;
}

class FLC_Updater {

    use FLC_Singleton;

    protected $version;
    protected $settings;
    protected $folder;

    protected function __construct() {

        $this->version = FLC_Plugin::version;
        $this->settings = FLC_Settings::instance();
        $this->folder = FLC_BASEPATH . 'includes/updates';
    }
    
    public function update() {

        $db_version = $this->settings->get('plug-in_version', '0');

        switch(version_compare($db_version, $this->version)) {
            case -1: $this->upgrade($db_version); break;
            case 1: $this->downgrade($db_version); break;
        }
        
        $this->settings->set('plug-in_version', $this->version);
    }

    private function upgrade($db_version) {

        $updates = glob("$this->folder/upgrades/*.php");
        usort($updates, function($a, $b) {
            $version_a = pathinfo($a, PATHINFO_FILENAME);
            $version_b = pathinfo($b, PATHINFO_FILENAME);
            return version_compare($a, $b);
        });
        foreach($updates as $update_file) {
            $version = pathinfo($update_file, PATHINFO_FILENAME);
            if(version_compare($db_version, $version, '<')) {
                require $update_file;
                $this->settings->set('plug-in_version', $db_version = $version);
            }
        }
    }

    private function downgrade($db_version) {
        
        $updates = glob("$this->folder/downgrades/*.php");
        usort($updates, function($a, $b) {
            $version_a = pathinfo($a, PATHINFO_FILENAME);
            $version_b = pathinfo($b, PATHINFO_FILENAME);
            return version_compare($a, $b);
        });
        foreach($updates as $update_file) {
            $version = pathinfo($update_file, PATHINFO_FILENAME);
            if(version_compare($db_version, $version, '>')) {
                require $update_file;
                $this->settings->set('plug-in_version', $db_version = $version);
            }
        }
    }
}
