<?php

// If this file is called directly, abort.
if (!defined('FLC_BASEPATH')) {
    die;
}

class FLC_Entity_User extends FLC_Entity {

    const name = 'user';
    
    private $insert_api = 'user/create';
    private $update_api = 'user/edit';
    private $delete_api = 'user/delete';

    protected function __construct() {

        add_action('show_user_profile', array($this, 'fields'), 10, 1);
        add_action('edit_user_profile', array($this, 'fields'), 10, 1);
        
        add_action('personal_options_update', array($this, 'saveFields'), 10, 1);
        add_action('edit_user_profile_update', array($this, 'saveFields'), 10, 1);
        
        add_action('user_register', array($this, 'save'), 10, 1);
        add_action('profile_update', array($this, 'save'), 10, 1);
        add_action('delete_user', array($this, 'delete'), 10, 1);
    }

    public function fields($user) {

        if(is_admin()) {
            ?>
            <table class="form-table">
                <tr>
                    <th>
                        <label for="flc_id"><?php esc_html_e('forma.lms ID'); ?></label>
                    </th>
                    <td>
                        <input type="number" name="flc_id" id="flc_id" value="<?php echo esc_attr(get_user_meta($user->ID, 'flc_id', true)); ?>" />
                        <br><span class="description"><?php esc_html_e('forma.lms ID', 'flc-forma-lms-connector'); ?></span>
                    </td>
                </tr>
            </table>
            <?php
        } else {
            ?>
            <input type="hidden" name="flc_id" id="flc_id" value="<?php echo esc_attr(get_user_meta($user->ID, 'flc_id', true)); ?>" />
            <?php
        }
    }

    public function saveFields($user_id) {

        if (empty($_POST['_wpnonce']) || !wp_verify_nonce($_POST['_wpnonce'], 'update-user_' . $user_id)) {
            return;
        }

        if (!current_user_can('edit_user', $user_id)) {
            return false;
        }

        update_user_meta($user_id, 'flc_id', $_POST['flc_id']);
    }

    public function save($user_id) {

        if(get_user_meta($user_id, 'flc_id', true)) {
            $this->update($user_id);
        } else {
            $this->insert($user_id);
        }
    }

    public function insert($user_id) {

        $params = $this->getUserParameters($user_id);
        $res = $this->callAPI($this->insert_api, $params);
        if(!empty($res['success'])) {
            add_user_meta($user_id, 'flc_id', $res['idst']);
        }
    }

    public function update($user_id) {

        $params = $this->getUserParameters($user_id);
        $this->callAPI($this->update_api, $params);
    }

    public function delete($user_id) {

        $params = $this->getUserParameters($user_id);
        $this->callAPI($this->delete_api, $params);
    }

    private function getUserParameters($user_id) {

        $user = get_userdata($user_id);

        $params = array(
            'userid'    => $user->data->user_login
          , 'email'     => $user->data->user_email
          , 'firstname' => get_user_meta($user_id, 'first_name', true)
          , 'lastname'  => get_user_meta($user_id, 'last_name', true)
          , 'ext_user'  => $user_id
          , 'ext_user_type' => 'wordpress'
        );

        return $params;
    }

    public static function getTitle() {

        return __('Users', 'flc-forma-lms-connector');
    }
}

add_action('flc.register-entity', array('FLC_Entity_User', 'register'), 10);
